/*** -*- Mode: Javascript; tab-width: 2;

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is Collabnet code.
The Initial Developer of the Original Code is Collabnet.

Portions created by Collabnet are
Copyright (C) 2000 Collabnet.  All
Rights Reserved.

Contributor(s): Pete Collins, Doug Turner, Brendan Eich, Warren Harris, Eric Plaster


 File API (The purpose of this file is to make it a little easier to do file IO from js) 

       file.js

 Function List

    // Constructor
    File(aPath)                      // creates the File object and sets the file path
    or File(nsILocalFile)         	 // The path may also be a chrome: or file: url.
	
    exists();                        // check to see if a file exists

    // file attributes
    permissions;                     // returns the files permissions
    dateModified;                    // returns the last modified Date 
    size;                            // returns the file size

    // file string manipulation
    path;                            // returns the path
    leaf;                            // leaf is the endmost file string ex: foo.html in /myDir/foo.html
    extension;                       // returns a file extension if there is one
    dirPath;                         // returns the dir part of a path

    // direct manipulation
    nsIFIle()                          // returns an nsIFile obj 

 Instructions:
				
	First include this following script tag in your xul file:
       	
       	<script type="application/x-javascript" src="chrome://xalib/content/XaLib.js"/>
	

		Then include this file in your js file:       	
		
		XaLib.include("chrome://xalib/content/IO/File/File.js"); // Always use a chrome url.

	To create a new File object call:
    	
    	var file = new File("/path/file.ext");
    	
	    The supplied path may be a native path to your OS, a chrome: url, or a file: url.
		
	To see if the file exists, call the exists() member.  This is a good check before going into some
	deep code to try and extract information from a non-existant file. However, an exception will be thrown 
	for attempted access to non-existent files.
************/
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");

XaLib.include("chrome://xalib/content/XPCOM/XPCOM.js");
XaLib.include("chrome://xalib/content/StringExtensions.js");

/****************** File Object Class *********************/

function File(aPath) {
	if (!aPath) 
		throw File.getMessage("File.PathNotSpecified", getStackTrace() );

	try {
		if (aPath instanceof Components.interfaces.nsILocalFile) {
			this.localFile = aPath;
		}
		else {
			this._path = aPath;
			
			aPath = File.URLToPath(aPath);		
			var aURI = File.pathToURL(aPath)
			this._URI = File._IOService.newURI(aURI, null, null);			

			if (aURI.startsWith("file:")) 
				this._localFile = new File._localFile(aPath);
		}
	}
	catch (e) {
		throw 'File: failed to initialize.\n' + e;
	}
	
} // constructor

// Initialize File
try {
	// Initialize File properties.

	File.DIRECTORY_TYPE			= 1;

	File._IOService = XPCOM.getService("@mozilla.org/network/io-service;1", "nsIIOService");

	File._localFile     = new Components.Constructor( "@mozilla.org/file/local;1", "nsILocalFile", "initWithPath");

	File._RegistryService = XPCOM.getService("@mozilla.org/chrome/chrome-registry;1", "nsIChromeRegistry");
	
	File._supportedProtocolsRE = /^file:|^chrome:|^jar:|^http:|^https:|^ftp:|^dir:|^rdf:/;
	File._mappedProtocolsRE = /^chrome:|^dir:/;
}
catch (e) {
	throw 'File: failed to initialize.\n' + e;
}

File.getMessage = function getMessage() {
	if (!File._messages)
		File._messages = new MessageFile("chrome://xalib/locale/IO/File/File.msg");

	return File._messages.getMessage(arguments);
}

/********************* PARENT DIR *****************************/
File.prototype.__defineGetter__('parent', function () {
	if (!this._localFile) throw File.getMessage("parent.LocalFilesOnly", this.path, getStackTrace()); 
	return new File(this._localFile.parent.path);
})

/********************* PATH *****************************/
File.prototype.__defineGetter__('path', function () {
	if (this._localFile)
		return this._localFile.path;
	return this._URI.spec;	
})


/********************* URL *****************************/
File.prototype.__defineGetter__('URL', function () {
	return File.pathToURL(this.path);
})

/********************* EXISTS ***************************/
File.prototype.exists = function () {
	if (!this._localFile) throw File.getMessage("exists.LocalFilesOnly", this.path, getStackTrace()); 
  		
	try {
		return this._localFile.exists();
	}
	catch(e) { 
		throw File.getMessage("exists.UnknownError", this.path, getStackTrace()); 
	}
}


/********************* LEAF *****************************/
File.prototype.__defineGetter__('leaf', function () {
	try  {
		if (this._URI instanceof Components.interfaces.nsIURL)
			return decodeURI(this._URI.fileName);
	}
	catch(e) {
		throw "File.leaf: Error: " + e;
	}
})


/********************* PERMISSIONS **********************/
File.prototype.__defineGetter__('permissions', function () {
	if(this.exists()) {
		return this._localFile.permissions.toString(8);
	}
	else
	  	throw "File.permissions: File " + this.path + " does not exist.";
})

/********************* MODIFIED *************************/
File.prototype.__defineGetter__('dateModified', function () {
  if(this.exists()) {
  	return new Date(this._localFile.lastModificationDate).toLocaleString();
  }
  else
  	throw "File.dateModified: File " + this.path + " does not exist.";
})

/********************* SIZE *****************************/
File.prototype.__defineGetter__('size', function ()  {
  if(this.exists()) {
	  try { 
		return this._localFile.fileSize;
	  }
	  catch(e) { 
			throw "File.size: Error: " + e;
	  }
  }
  else
  	throw "File.size: File " + this.path + " does not exist.";
})

/********************* EXTENSION ************************/
File.prototype.__defineGetter__('extension', function () {
	if(this.exists()) {
		try { 
			var leafName  = this._localFile.leafName; 
			var dotIndex  = leafName.lastIndexOf('.'); 
			return (dotIndex >= 0) ? leafName.substring(dotIndex+1) : ""; 
		}
		catch(e) {
			throw "File.extension: Error: " + e;
		}
	}
	else
	  	throw "File.extension: File " + this.path + " does not exist.";

})

/********************* DIRPATH **************************/
File.prototype.__defineGetter__('dirPath', function () {
	if(this.exists()) {
		try { 
			if(this._localFile)
			  return this._localFile.parent.path;

			//TODO
		}
		catch (e) { 
			throw "File.dirPath: Error: " + e;
		}
	}
	else
		throw "File.dirPath: File " + this.path + " does not exist.";
	
})

/********************* nsIFILE **************************/
File.prototype.__defineGetter__('localFile', function () {
	return this._localFile;
})
File.prototype.__defineSetter__('localFile', function (aLocalFile) {
	if (aLocalFile instanceof Components.interfaces.nsILocalFile) {
		this._localFile = aLocalFile;
		this._path = aLocalFile.path;
		this._URI = File._IOService.newURI(File.pathToURL(this._path), null, null);			
	}
	else 
		throw ("setLocalFile.NotLocalFile", getSTackTrace()); 
})

File.prototype.isFile = function() {
	try {
		if (this._localFile)
			return this._localFile.isFile();
		else
			return false;
	}
	catch (e) { 
		throw "File.isFile: Error: " + e;
	}
}
File.prototype.isReadable = function() {
	try {
		if (this._localFile)
			return this._localFile.isReadable();
		else
			return true;
	}
	catch (e) { 
		throw "File.isReadable: Error: " + e;
	}
}


File.prototype.isWritable = function() {
	try { 
		if (this._localFile)
			return this._localFile.isWritable();
		else
			return false;
	}
	catch (e) { 
		throw "File.isWritable: Error: " + e;
	}
}
File.prototype.isDirectory = function() {

	try { 
		if (this._localFile)
			return this._localFile.isDirectory();
		else
			return false;
	}
	catch (e) { 
		throw "File.isDirectory: Error: " + e;
	}
}

File.prototype.isSymlink = function() {
	try { 
		if (this._localFile)
			return this._localFile.isSymlink();
		else
			return false;
	}
	catch (e) { 
		throw "File.isSymlink: Error: " + e;
	}
}
File.prototype.isSpecial = function() {
	try { 
		if (this._localFile)
			return this._localFile.isSpecial();
		else
			return false;
	}
	catch (e) { 
		throw "File.isSpecial: Error: " + e;
	}
}
File.prototype.isHidden = function() {
	try { 
		if (this._localFile)
			return this._localFile.isHidden();
		else
			return false;
	}
	catch (e) { 
		throw "File.isHidden: Error: " + e;
	}
}



/**
 * Convenience function for converting chrome to file: URL 
**/
File.chromeToFileURL= function(chromeURL)  {
	var  uriObj = File._IOService.newURI(chromeURL,null,null);  
    return File._RegistryService.convertChromeURL(uriObj).toString();
}
/**
 * Convenience function for converting URL to native path. 
**/
File.URLToPath = function(URL) {
	var path = URL;
	var arr = File._supportedProtocolsRE.exec(URL);
	var protocol = "native:";
	if (arr)
		protocol = arr[0];
	switch (protocol) {
	case "native":
		break;
	case "dir:":	
		// Auto load directory service.
		if (!File.DIRECTORYSERVICE_LOADED) {
			XaLib.include("chrome://xalib/content/IO/File/DirectoryService.js");
			File.DIRECTORYSERVICE_LOADED = true;
		}
		path = File.dirURLToPath(URL); // returns native path.
		break;
	case "chrome:":	
		path = File.chromeToFileURL(URL); // returns file:, jar:
		if (path.substr(0, 4) == "jar:")
			return path;
		// drop thru
	case "file:":	
		if (path.substr(0,8) == "file:///")
			path = path.substr(8);
		else if (URL.substr(0,7) == "file://")
			path = path.substr(7);
		break;
	default:
		return URL; // http: https: ftp:
	}
	path = decodeURI(path);
	//path = path.replace(/%20/g," "); // Replace space references.
	if (File.pathSeparator == "\\") { 
		path = path.replace(/\//g,"\\"); // Replace forward by retarded slashes.
		path = path.replace(/\|/g,":"); // Replace drive separator.
	}	
	return path;
}
/**
 * Convenience function for converting native path to file URL. 
**/
File.pathToURL = function(path) {
	if (File._supportedProtocolsRE.test(path))
		return path;

	var URL = path;
	if (File.pathSeparator == "\\") { 
		URL = URL.replace(/\\/g,"/"); // Replace retarded slashes by forward slashes.
		URL = URL.replace(/:/g,"|"); // Replace drive separator.
	}	
	URL = encodeURI(URL);
	URL = URL.replace(/%7c/gi , "|"); // encode goes too far.

	return "file://" + URL;
}

File.dirPath = function(path) {
	if (!path)
		return null;
	var sep;
	if (path.startsWith("chrome:") || path.startsWith("file:"))
		sep = "/";
	else
		sep = File.pathSeparator; 
	var i = path.lastIndexOf(sep);
	if (i > -1)
		return path.substring(0,i);
	else
		return path;
}
File.leaf = function
// Convenience function to get a native path filename.extension.
(	path	//string the native path
)			//string: the filename.extension
{
	if (!path)
		return null;
	var dirPath = File.dirPath(path);
	return path.substr(dirPath.length); 
}
File.__defineGetter__('pathSeparator', function () {
	return (navigator.appVersion.indexOf("Win") != -1)? "\\" : "/"; 
})
