/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");

function TokenDef(name,args) {
	return new Tokenizer.TokenDef(name, args);
}
function Token() {
}

function Tokenizer(lexDefText) {
	var lexDef;
	eval(lexDefText);
	this.states = {};
	this.states.__proto__ = null; // use as hash map.
	
	var stateFixup = []; // Array of tokenDefs which refer to other states.
	for (var stateName in lexDef.states) {
		this.states[stateName] = new Tokenizer.State(stateName,lexDef, stateFixup);
	}
	
	// Fixup nextState references to refer to actual state object.
	for (var i=0; i<stateFixup.length; i++) {
		var tokenDef = stateFixup[i];
		if (this.states[tokenDef.nextState]) 
			tokenDef.nextState = this.states[tokenDef.nextState];			
		else	
			throw "nextstate:" + tokenDef.nextState + " refers to undefined state.";
	}
	
	// Define end of file token.	
	this.EOF = new TokenDef("EOF");
}
Tokenizer.TokenDef = function(name,args) {
	this.name = name;
	if (args) {
		// Copy TokenDef properties from args object.
		for (var argName in args) {
			this[argName] = args[argName];
		}
	}
}

Tokenizer.State = function(stateName, lexDef, stateFixup) {
	this.stateName = stateName;
	this.tokenDefs = {};
	this.tokenDefs.__proto__ = null; // use as hash table.

	this.variable = [];		// tokenDefs having variable length results.
	this.variable.push(""); // dummy entry to lineup with regex results array.
	
	var isMacroDef = /<([^>]+)>/g; // match macro def use.
	
	// Cache text representation of RegExp.
	var macros = lexDef.macros || {};
	for (var macroName in macros) {
		var body = macros[macroName].toString();
		body = body.substring(1, body.length - 1 ); // remove /.../
		macros[macroName].body = body;
	}

	var re = "";
	var orSep = "";
	

	var tokenDefs = lexDef.states[stateName];
	for (var i=0; i<tokenDefs.length; i++) {
		var tokenDef = tokenDefs[i];
		
		if (tokenDef.id)
			this.tokenDefs[tokenDef.id] = tokenDef;
		else if (tokenDef.variable)
			this.variable.push(tokenDef);
		else if (tokenDef.name)
				this.tokenDefs[tokenDef.name] = tokenDef;
		else			
			throw "TokenDef must have either 'id:aString' or 'variable:true'."; 
			
		// Memo tokens which refer to other states.	
		if (tokenDef.nextState) {
			stateFixup.push(tokenDef);
		}
		
		if (tokenDef.thenMatch) {
			tokenDef.thenMatch.global = true;
			tokenDef.thenMatch.multiline = true;
		}


		if (tokenDef.match) {
			var match = tokenDef.match.toString();
			match = match.substring(1,match.length-1); // drop /.../ from re string.
			// replace any macros.
			var result = "X";
			var newMatch = match;
			isMacroDef.lastIndex = 0;
			while (result) {
				result = isMacroDef.exec(newMatch);
				if (result) {
					for (var j=1; j<result.length; j++) {
						var macroName = result[j];
						if (macros[macroName]) {
							newMatch = newMatch.replace(new RegExp("<" + macroName + ">","g"), macros[macroName].body);
						}
						else {
							throw "Reference to undefined macro:" + macroName;
						}
					}
					isMacroDef.lastIndex = 0; // Allow included macros to include macros.
				}
			}
			match = newMatch;
			
			if (tokenDef.variable)
				re += orSep + "(" + match + ")"; // group variable length patterns so we can find them.
			else	
				re += orSep +  match;
			orSep = "|"
		}
	}
	
	this.regex = new RegExp(re,"gm");
}

Tokenizer.prototype.init = function(text) {
	this.text = text;
	var stateName = "normal";
	this.state = this.states[stateName];
	this.regex = this.state.regex;
	this.regex.lastIndex = 0;
}

Tokenizer.prototype.nextToken = function(token) {
	var regex = this.regex;
	var state = this.state;
	var text = this.text;
	while (regex.lastIndex < text.length) {
		var	results = regex.exec(text); 
		if (!results)
			break;
		token.val = results[0];
		token.offset = regex.lastIndex - token.val.length;
		var t = null;
		t = state.tokenDefs[token.val];
		if (!t) {
			// lookup variable length patterns for match.
			// first match is taken ==> Put more complex patterns first in definitions.
			for (var i=1; i<results.length; i++) {
				if (results[i]) {
					t = state.variable[i];
					break;
				}
			}
		}

		if (t) {
			if (t.ignore)
				continue;
			token.type = t;	
			if (t.thenMatch) {
				t.thenMatch.lastIndex = regex.lastIndex;
				results = t.thenMatch.exec(text);
				token.val += results[0];
				token.offset = t.thenMatch.lastIndex - token.val.length;
				regex.lastIndex = t.thenMatch.lastIndex;
			}
			if (t.nextState) {
				this.currentState = t.nextState;
				var i = regex.lastIndex;
				this.state = this.states[this.currentState];
				this.regex = regex = state.regex;
				regex.lastIndex = i; // resume where previous regex finished.
			}
		}
		else {
			// error
			var i = regex.lastIndex;
			this.state = this.states["normal"];
			this.regex = regex = state.regex;
			regex.lastIndex = i; // resume where previous regex finished.
		}
		return;
	}
	token.type = this.EOF;
	token.val = null;
	token.offset = text.length;
}

