/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
XaLib.include("chrome://xalib/content/RDF/RDF.js");

const NS_ERROR_RDF_BASE = 0x804f0000; // XXX is this right?
const NS_RDF_CURSOR_EMPTY = NS_ERROR_RDF_BASE + 1;
const NS_RDF_NO_VALUE = NS_ERROR_RDF_BASE + 2;
const NS_RDF_ASSERTION_ACCEPTED = Components.results.NS_OK;
const NS_RDF_ASSERTION_REJECTED = NS_ERROR_RDF_BASE + 3;


function FactStore
/* This class provides a Facade to an RDF DataStore. 
* WARNING: This class is NOT a DataStore and should not be passed to XPCOM components expecting
* a nsIDataSource. Pass: aFactStore.dataStore instead.
* Notes: 
* The terms Subject, Predicate, and Object replace the non-standard Mozilla terms
* Source, Property and Target respectively.
* The use of the Truth feature has not been implemented. I can't find a single example of its use.
*/
(
	dataSource //nsIRDFDataSource: The dataSource to be wrapped.
)
{
  this._dataSource = dataSource;
}
FactStore.prototype.__defineGetter__('dataSource', function () {
	return this._dataSource;
})
FactStore.prototype.__defineGetter__('URI', function () {
	return this._dataSource.URI;
})


FactStore.prototype.hasAssertion = function
// Query whether an assertion exists in this graph.
(
	subject,	//nsRDFResource: The subject. 
	predicate, 	//nsIRDFResource: The predicate.  
	object  	//oneof(nsIRDFNode, string, integer, Date): The object
)
{
	return this._dataSource.HasAssertion(subject, predicate, RDF.node(object), true);
}

FactStore.prototype.subject = function
/* Find an RDF resource that is related to the given <tt>object</tt> by
* the specified <tt>predicate</tt>.
* Query Pattern: (?,predicate, object)
*/
(
	predicate, 	//nsIRDFResource: the predicate
	object		//oneof(nsIRDFNode, string, integer, Date): the object 
) 				//?nsIRDFResource
{
	var subject =  this._dataSource.GetSource(predicate, RDF.node(object), true);
	if (Components.returnCode == NS_RDF_NO_VALUE)
		return null;
	else
		return subject;
	 
}
FactStore.prototype.subjects = function
/* Find RDF subject resources that are related to the given <tt>object</tt>
* by the specified <tt>predicate</tt>.
* The returned iterator can be used as follows:
* <code>
*	var iterator = aDS.getSubjects(aPredicate, aObject);
*	var resource;
*   while ((resource = iterator() )) { // Use double parentheses to suppress strict errors.
*	}
* </code>
* Query Pattern: (*,predicate, object)
*/
(
	predicate, 	//nsIRDFResource: the predicate
	object		//oneof(nsIRDFNode, string, integer, Date): the object 
) 				//function: An iterator. The iterator returns ?nsIRDFResource objects.
{
	var subjectIterator = this._dataSource.GetSources(predicate, RDF.node(object), true);
	if (Components.returnCode != Components.results.NS_OK)
		throw Components.lastError;
	// iterator closure:	
	var iterator = function () {
		if (subjectIterator.hasMoreElements()) {
			var resource = subjectIterator.getNext()
			resource = resource.QueryInterface(Components.interfaces.nsIRDFResource);
			return resource;
		}
		else
			return null;
	}
	return iterator;
}

FactStore.prototype.object = function
/* Find RDF object node that is related to the given <tt>subject</tt>
* by the specified <tt>predicate</tt>.
* Query Pattern: (subject,predicate, ?)
*/
(
	subject,	//nsIRDFResourse: the subject
	predicate 	//nsIRDFResource: the predicate
)				//?oneof(nsIRDFResource, string, Date, integer):
{
	 return RDF.value(this._dataSource.GetTarget(subject, predicate, true));
}

FactStore.prototype.objects = function
/* Find RDF object node that is related to the given <tt>subject</tt>
* by the specified <tt>predicate</tt>.
* <code>
*	var iterator = aDS.getObjects(aPredicate, aObject);
*	var object;
*   while ((object = iterator() )) { // Use double parentheses to suppress strict errors.
		var val = object.Value;
*	}
* </code>
* Query Pattern: (subject,predicate, *)
*/
(
	subject,	//nsIRDFResourse: the subject
	predicate 	//nsIRDFResource: the predicate
)				//funtion: An iterator. The iterator returns ?oneof(nsIRDFResource, string, Date, integer)
{
	var objectIterator = this._dataSource.GetTargets(subject, predicate, true);
	if (Components.returnCode != Components.results.NS_OK)
		throw Components.lastError;
	// iterator closure:	
	var iterator = function () {
		if (objectIterator.hasMoreElements()) {
			var object = objectIterator.getNext()
			return RDF.value(object)
		}
		else
			return null;
	}
	return iterator;
}

FactStore.prototype.hasPredicateIn = function
// Does a <tt>predicate</tt> point into an <tt>object</tt>.
// Query Pattern: ?(_,predicate,object)
(

	predicate,	//nsIRDFResource: The predicate
	object		//oneof(nsIRDFNode, string, integer, Date): the object 
)				//boolean: 
{
	return this._dataSource.hasArcIn(FactStore(object), predicate);
}

FactStore.prototype.predicatesIn = function
/* Get an iterator for the unique list of <tt>predicates</tt> that point into an <tt>object</tt>.
* <code>
*	var iterator = aDS.predicatesIn(aObject);
*	var predicate;
*   while ((predicate = iterator() )) { // Use double parentheses to suppress strict errors.
*	}
* </code>
* Query Pattern: (_ ,* , object)
*/
(
	object 	//oneof(nsIRDFNode, string, integer, Date): The object
)			//funtion: An iterator. The iterator returns ?nsIRDFResource of type predicate.
{
	var predicateIterator = this._dataSource.ArcLabelsIn(RDF.node(object));
	if (Components.returnCode != Components.results.NS_OK)
		throw Components.lastError;
	// iterator closure:	
	var iterator = function () {
		if (predicateIterator.hasMoreElements()) {
			var predicate = predicateIterator.getNext()
			return predicate.QueryInterface(Components.interfaces.nsIRDFResource);
		}
		else
			return null;
	}
	return iterator;
}

FactStore.prototype.hasPredicateOut = function
// Does a <tt>predicate</tt> point from a <tt>subject</tt>.
// Query Pattern: ?(subject,predicate,_)
(
	subject,	//nsIRDFResource: The subject
	predicate	//nsIRDFResource: The predicate
)				//boolean:
{
	return this._dataSource.hasArcOut(subject, predicate);
}

FactStore.prototype.predicatesOut = function
/* Get an iterator for the unique list of <tt>predicates</tt> that point out from an <tt>subject</tt>.
* <code>
*	var iterator = aDS.predicatesOut(aSubject);
*	var predicate;
*   while ((predicate = iterator() )) { // Use double parentheses to suppress strict errors.
*	}
* </code>
* Query Pattern: (_ ,* , object)
*/
(
	subject		//nsIRDFResource: The subject
)				//funtion: An iterator. The iterator returns ?nsIRDFResource of type predicate.
{
	var predicateIterator = this._dataSource.ArcLabelsOut(RDF.node(object));
	if (Components.returnCode != Components.results.NS_OK)
		throw Components.lastError;
	// iterator closure:	
	var iterator = function () {
		if (predicateIterator.hasMoreElements()) {
			var predicate = predicateIterator.getNext()
			return predicate.QueryInterface(Components.interfaces.nsIRDFResource);
		}
		else
			return null;
	}
	return iterator;
}

FactStore.prototype.resources = function
// Get an iterator for all resources in the DataStore.
(
)	//function: An iterator returns ?nsIRDFResource.
{
	var resourceIterator = this._dataSource.GetAllResources();
	if (Components.returnCode != Components.results.NS_OK)
		throw Components.lastError;
	// iterator closure:	
	var iterator = function () {
		if (resourceIterator.hasMoreElements()) {
			var resource = resourceIterator.getNext()
			return resource.QueryInterface(Components.interfaces.nsIRDFResource);
		}
		else
			return null;
	}
	return iterator;
}


FactStore.prototype.assert = function
// Add an assertion to the graph.
(
	subject, 	//nsRDFResource: The subject.
	predicate,	//nsIRDFResource: The predicate. 
	object		//oneof(nsIRDFNode, string, integer, Date): The object
)				//void:
{
	 this._dataSource.Assert(subject, predicate, RDF.node(object), true);
}

FactStore.prototype.retract = function
// Remove an assertion from the graph.

(
	subject, 		//nsRDFResource: The subject.
	predicate,		//nsIRDFResource: The predicate.  
	object			//oneof(nsIRDFNode, string, integer, Date): The object
)				//void:
{
	 this._dataSource.Unassert(subject, predicate, RDF.node(object), true);
}

FactStore.prototype.replace = function
// Replace the object on a specified assertion.
// This is equivalent to:
//		retract(subject,predicate, object);
//		assert(subject, predicate, newObject); 
(
	subject,	//nsRDFResource: The subject.
	predicate,	//nsIRDFResource: The predicate.  
	object,		//oneof(nsIRDFNode, string, integer, Date): The original object
	newObject	//oneof(nsIRDFNode, string, integer, Date): The replacement object
)				//void:
{
	 this._dataSource.Change(subject, predicate, RDF.node(object), RDF.node(newObject));
}

FactStore.prototype.move = function
// Replace the subject on a specified assertion.
// This is equivalent to:
//		retract(subject,predicate, object);
//		assert(newSubject, predicate); 
(
	subject,	//nsRDFResource: The oiginal subject. 
	predicate, 	//nsIRDFResource: The predicate.  
	object, 	//oneof(nsIRDFNode, string, integer, Date): The object
	newSubject 	//nsRDFResource: The replaement subject. 
)
{
	throw this._dataSource.Move(oldSubject, newSubject, predicate, RDF.node(object)); 
}

FactStore.prototype.AddObserver = function(aObserver) {
	 this._dataSource.AddObserver(aObserver);
}

FactStore.prototype.RemoveObserver = function(aObserver) {
	 this._dataSource.RemoveObserver(aObserver);
}

