XaLib.include("chrome://xalib/content/XPCOM/XPCOM.js");
XaLib.include("chrome://xalib/content/IO/File/File.js");
//XaLib.include("chrome://xalib/content/StringExtensions.js"); // Already in File.


function RDF() {
}
try {
	RDF.RDFService = XPCOM.getService("@mozilla.org/rdf/rdf-service;1",
		"nsIRDFService");
	RDF.InMemoryDataSource = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"].
							createInstance(Components.interfaces.nsIRDFInMemoryDataSource);	
}
catch (e) {
	throw "RDF failed to initialize.";
}
RDF.getMessage = function getMessage() {
	if (!RDF._messages)
		RDF._messages = new MessageFile("chrome://xalib/locale/RDF/RDF.msg");

	return RDF._messages.getMessage(arguments);
}

RDF.getDataSource = function
(
	URL	/*string: The URI which locates the datasource to be loaded.
				  A native path may also be supplied.
		*/
)		//nsIRDFDataSource: the requested Datasource.
{
	//Note: DataSources retrieved using a chrome url are not updatable (I don't know why),
	//the above conterts dir:, chrome: file: urls to native paths.
	const NSRESULT_RDF_SYNTAX_ERROR = 0x804e03f7;

	try {
		var path = File.URLToPath(URL); //Noop if already a native path. but also maps chrome: and dir:

		return RDF.RDFService.GetDataSourceBlocking(File.pathToURL(path));
	} catch (e if (e.result == NSRESULT_RDF_SYNTAX_ERROR)) {
		throw RDF.getMessage("getDataSource.SyntaxError", URL, e, getStackTrace());
	} catch (e) {
		throw RDF.getMessage("getDataSource.Error",  URL, e, getStackTrace());
	}
}

RDF.unregister = function
(
	URI
)
{
	if (URI && typeof(URI) == "string")
		RDF.RDFService.UnregisterDataSource(URI);	
}


RDF.resource = function
/** Construct an RDF resource from a URI string.
* <tt>nsIRDFSerivce</tt> caches resources that are in-use, so multiple calls to <tt>getResource()</tt>
* for the same <tt>URI</tt> will return identical pointers.
* Use <tt>FindResource</tt> to find out whether the resource already exists.
**/
(

	URI	//string: URN or URI of resource. A native path is not accepted.
)

{
	return RDF.RDFService.GetUnicodeResource(URI);
}

RDF.anonymousResource = function
/** Construct an anonymouse RDF resource.
* <tt>nsIRDFSerivce</tt> returns a unique resouse every time this method is called.
**/
()
{
	return RDF.RDFService.GetAnonymousResource(URI);
}
RDF.string = function
// Construct an RDF literal from a string.
(

	aValue	//oneof(string, nsIRDFLiteral): The literal value.
			//returns: nsIRDFLiteral.
) 
{
	if (typeof(aValue) == "string")
		return  RDF.RDFService.GetLiteral(aValue);
	else	
		throw "invalid string.";
}

RDF.date = function
// Construct an RDF date from a date value.
(
	aValue	//oneof(string: The date value. May be a Date object, a number (Date.valueOf), or a Date string.
			//returns: nsIRDFDate
) 
{
	if (aValue instanceof Date)
		return  RDF.RDFService.GetDateLiteral(aValue.valueOf());
	else
			throw "invalid date";
}

RDF.integer = function
// Construct an RDF date from a integer value.
(
	aValue	//string: The Date value.
			//returns: nsIRDFInt 
) 
{
	if (typeof(aValue) == "number")
		return  RDF.RDFService.GetIntLiteral(aValue);
	else
		throw "invalid integer";
}

RDF.registerDataSource = function
// Register a Datasource with the RDFService for later access.
(
)
{
}


RDF.node = function 
// Construct an RDF node from a string, integer or Date value.
// Values which implement nsIRDFNode are passed through unchanged.
(
	aValue,		//oneof(nsIRDFNode, string, number, Date): The value.
	NSResolver  //?NDResolver: An object which conforms to the RDF.NSResolver interface.
				// if no value is supplied the RDF.DefaultNSResolver is used.
) 			//nsIRDFNode: 
{
	if (aValue instanceof Components.interfaces.nsIRDFNode)
		return aValue;
	else if (typeof(aValue) == "string") {
		var resolver = NSResolver || RDF.DefaultNSResolver;
		if (resolver.isEncodedResource(aValue))
			return RDF.resource(resolver.decode(aValue));			
		else	
			return RDF.RDFService.GetLiteral(aValue);
	}
	else if (typeof(aValue) == "number")
		return  RDF.RDFService.GetIntLiteral(aValue);
	else if (aValue instanceof Date)
		return  RDF.RDFService.GetDateLiteral(aValue.valueOf());
	throw "Invalid RDF literal";
}

RDF.value = function
(
	node,	//nsIRDFNode: The Node whose value is to be determined.
	NSResolver  //?NDResolver: An object which conforms to the RDF.NSResolver interface.
				// if no value is supplied the RDF.DefaultNSResolver is used.

)			//oneof(nsIRDFResource, string, integer, Date)
{
	if (!node)
		return null;
	try {	
		var result = node.QueryInterface(Components.interfaces.nsIRDFLiteral);
		return result.Value;
	}
	catch (e) {}
	try {
		result = node.QueryInterface(Components.interfaces.nsIRDFResource);
		var resolver = NSResolver || RDF.DefaultNSResolver;
		return resolver.encode(result.ValueUTF8);
	}
	catch (e) {}
	try {
		result = node.QueryInterface(Components.interfaces.nsIRDFInt);
		return result.Value;
	}
	catch (e) {}
	try {
		result = node.QueryInterface(Components.interfaces.nsIRDFDate);
		return new Date(result.Value);
	}
	catch (e) {}
	//result = node.QueryInterface(Components.interfaces.nsIRDBlobLiteral);
	//if (result) return result;
	throw RDF.getMessage("value.UnknownRDFNodeType", getStackTrace());
}
RDF.QINode = function
(
	node	//nsIRDFNode: The Node whose value is to be determined.
)			//?oneof(nsIRDFResource, nsIRDFLiteral, nsIRDFInt, nsIRDFDate)
{
	if (!node)
		return null;
	try {	
		return node.QueryInterface(Components.interfaces.nsIRDFLiteral);
	}
	catch (e) {}
	try {
		return node.QueryInterface(Components.interfaces.nsIRDFResource);
	}
	catch (e) {}
	try {
		return node.QueryInterface(Components.interfaces.nsIRDFInt);
	}
	catch (e) {}
	try {
		return node.QueryInterface(Components.interfaces.nsIRDFDate);
	}
	catch (e) {}
	//result = node.QueryInterface(Components.interfaces.nsIRDBlobLiteral);
	//if (result) return result;
	throw RDF.getMessage("QINode.UnknownRDFNodeType", getStackTrace());

}

RDF.NSResolver = function
/* Constructor for RDF resource names which can be used for Namespace resolution.
* A resource name can have the following format:
* [startDelimiter][Namespace:]URI[endDelimiter]
* e.g. if Namespace <tt>rdf</tt> is registered as "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
* Then the following URI "&lt;rdf:myLabel>" is resolved as http://www.w3.org/1999/02/22-rdf-syntax-ns#msLabel"
*/
(
	startDelimiter, // The string which starts a resource name (defaults to "<").
	endDelimiter	// The string which terminates a resource name (defaults to ">"). 
)	
{
	this.namespaceMap = {}; // namespaces available for resolution
	this.prefixMap = {}; // prefix names available for resolution
	// Chain up to default resolver.
	if (RDF.DefaultNSResolver != undefined) {
		this.namespaceMap.__proto__ = RDF.DefaultNSResolver.namespaceMap;
		this.prefixMap.__proto__ = RDF.DefaultNSResolver.prefixMap;
	}
	if (startDelimiter == undefined)
		this.startDelimiter = "<";
	if (endDelimiter == undefined)
		this.endDelimiter = ">";
}

RDF.NSResolver.prototype.isEncodedResource = function
(
	encodedResource	//?string: The encoded resource name.
)	//boolean:
{
	if (encodedResource && encodedResource.startsWith(this.startDelimiter) && 
			encodedResource.endsWith(this.endDelimiter))
		return true;
	else
		return false;
}

RDF.NSResolver.prototype.decode = function
// Resolve an encoded resource name to a full resource name.
(
	encodedResource	//?string: The encoded resource name.
)	//?string: full resource name.
{
	if (!encodedResource)
		return encodedResource;
	if (!encodedResource.startsWith(this.startDelimiter) || 
			!encodedResource.endsWith(this.endDelimiter)) 
		return encodedResource;			
	// remove delimiters.
	encodedResource = encodedResource.substring(this.startDelimiter.length, 
									encodedResource.length - this.endDelimiter.length);
	var i = encodedResource.indexOf(":");
	if (i > 0) {
		var NSPrefix = encodedResource.substring(0,i);
		var NS = this.prefixMap[NSPrefix];
		if (NS != undefined)
			encodedResource = NS + encodedResource.substr(i+1);
	}
	return encodedResource;
}
RDF.NSResolver.prototype.encode = function
// Encode full resource name to an encoded resource name.
(
	resourceURI	//?string: The encoded resource name.
)	//?string: encoded resource name.
{
	if (!resourceURI)
		return null;
	var i = resourceURI.indexOf("#");
	if (i > 0) {
		var namespaceURI = resourceURI.substr(0,i+1);
		var prefix = this.namespaceMap[namespaceURI];
		if (prefix)
			return this.startDelimiter + prefix + ":" + resourceURI.substr(namespaceURI.length) + this.endDelimiter;
	}
	else if (resourceURI.substr(0,4) == "urn:") {
		namespaceURI = resourceURI;
		do {
			i = namespaceURI.lastIndexOf(":");
			if (i > 0) {
				namespaceURI = namespaceURI.substr(0,i);
				prefix = this.namespaceMap[namespaceURI];
				if (prefix) {
					return this.startDelimiter + prefix + resourceURI.substr(namespaceURI.length) + this.endDelimiter;
				}
			}
			
		} while (i>0);
	}
	return this.startDelimiter + resourceURI + this.endDelimiter;
}

RDF.NSResolver.prototype.AddNamespace = function
// Add a namespace to the resolver.
(

	prefix,			//string: The namespace prefix. Using urn or a protocol name would not be a good idea.
	namespaceURI	//string: The full namespace uri including the trailing #.
)
{
	this.prefixMap[prefix] = namespaceURI;
	this.namespaceMap[namespaceURI] = prefix;
}
RDF.NSResolver.prototype.RemoveNamespace = function
// Remove a namespace prefix from the resolver.
(
	prefix	//string: The namespace prefix. 
)
{
	var namespaceURI = this.prefixMap[prefix];
	delete this.prefixMap[prefix];
	delete this.namespaceMap[namespaceURI];
}

RDF.DefaultNSResolver = new RDF.NSResolver();
RDF.DefaultNSResolver.AddNamespace("rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
RDF.DefaultNSResolver.AddNamespace("rdfs", "http://www.w3.org/2000/01/rdf-schema#");
RDF.DefaultNSResolver.AddNamespace("xsd", "http://www.w3.org/2001/XMLSchema#");
RDF.DefaultNSResolver.AddNamespace("CHROME", "http://www.mozilla.org/rdf/chrome#");
RDF.DefaultNSResolver.AddNamespace("NS", "http://home.netscape.com/NC-rdf#");