/*
	An RDFTemplate is a parameterized rdf document which is used to add boilerplate triples
	to a Datastore.
*/
XaLib.include("chrome://xalib/content/RDF/RDF.js");

function RDFTemplate(DSTemplate, DSOutput) {
	if (typeof(DSTemplate) == "string")
		this._DSTemplate = RDF.getDataSource(DSTemplate);
	else if (DSTemplate instanceof Components.interfaces.nsIRDFDataSource)
		this._DSTemplate = DSTemplate;			
	if (typeof(DSOutput) == "string")
		this._DSOutput = RDF.getDataSource(DSOutput);
	else if (DSOutput instanceof Components.interfaces.nsIRDFDataSource)
		this._DSOutput = DSOutput;			

	this._load();
	this._containers = {};
}
RDFTemplate.prototype.__defineGetter__('DSTemplate', function () {
	return this._DSTemplate;
})
RDFTemplate.prototype.__defineGetter__('DSOutput', function () {
	return this._DSOutput;
})

RDFTemplate._CUtils = XPCOM.getService("@mozilla.org/rdf/container-utils;1", "nsIRDFContainerUtils");

RDFTemplate.Triple = function(subject,predicate,object)
{
	this.subject = subject;
	this.predicate = predicate;
	this.object = object;
}
RDFTemplate.Var = function(name)
{
	this.name = name;
}

RDFTemplate.prototype._load = function
// Load all triples into array.
// Note, and replace all variable references by entries in a variable map.
(

)

{
	// load all triples.
	this._vars = {};
	this._triples = [];
	this._containers = {}; // used on output.
	
	var itrSubj = this._DSTemplate.GetAllResources();
	while(itrSubj.hasMoreElements()) {
		var subject = itrSubj.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
		var itrPred = this._DSTemplate.ArcLabelsOut(subject);
		while(itrPred.hasMoreElements()) {
			var predicate = itrPred.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
			// no need to store nextVal triples.
			if (predicate.ValueUTF8 == RDFTemplate.NEXTVAL)
				continue;
			var itrObj = this._DSTemplate.GetTargets(subject,predicate, true);
			while(itrObj.hasMoreElements()) {
				var object = itrObj.getNext();
				var triple = new RDFTemplate.Triple(subject, predicate, object);
				this._triples.push(triple);
				
				// Detect namespaced variable references and replace with variables.
				var aVar = this.putVarIf(subject.ValueUTF8);
				if (aVar) {
					triple.subject = aVar;
				}	
				aVar = this.putVarIf(predicate.ValueUTF8);
				if (aVar) triple.predicate = aVar;
				if (object instanceof Components.interfaces.nsIRDFResource) {
					aVar = this.putVarIf(object.ValueUTF8);
					if (aVar) {
						triple.object = aVar;
						triple.objIsResource = true;
					}
				}
				else if (object instanceof Components.interfaces.nsIRDFLiteral) {
					aVar = this.putVarIf(object.Value);
					if (aVar) triple.object = aVar;
				}
				
				if (predicate.ValueUTF8 == RDFTemplate.INSTANCEOF) {
					if (RDFTemplate._CUtils.IsSeq(this._DSTemplate, subject))
						triple.makeContainer = RDFTemplate.prototype.makeSeq;
					else if (RDFTemplate._CUtils.IsBag(this._DSTemplate, subject))
						triple.makeContainer = RDFTemplate.prototype.makeBag;
					else if (RDFTemplate._CUtils.IsAlt(this._DSTemplate, subject))
						triple.makeContainer = RDFTemplate.prototype.makeAlt;
				}	
				if (RDFTemplate._CUtils.IsOrdinalProperty(predicate))
					triple.appendObject = RDFTemplate.prototype.appendObject;
			}	
		}
	}
}

RDFTemplate.prototype.makeSeq = function (template, subject) {
	template._containers[subject.ValueUTF8] = RDFTemplate._CUtils.MakeSeq(template._DSOutput, subject);
}
RDFTemplate.prototype.makeBag = function (template, subject) {
	template._containers[subject.ValueUTF8] = RDFTemplate._CUtils.MakeBag(template._DSOutput, subject);
}
RDFTemplate.prototype.makeAlt = function (template, subject) {
	template._containers[subject.ValueUTF8] = RDFTemplate._CUtils.MakeAlt(template._DSOutput, subject);
}
RDFTemplate.prototype.appendObject = function (template, subject, object) {
	var cont = template._containers[subject.ValueUTF8];
	if (cont) 
		cont.AppendElement(object);
	else 
		throw "No container for subject:" + subject.ValueUTF8;
}

RDFTemplate.prototype.putVarIf = function(value) {
	if (value.startsWith(RDFTemplate.VAR)){ // var: in literal
		var varName = value.substr(RDFTemplate.VAR.length);
		var aVar = this._vars[varName];
		if (!aVar) {
			aVar = new RDFTemplate.Var(varName);
			this._vars[varName] = aVar;
		}
		return aVar;
	}
	else if (value.startsWith(RDFTemplate.NS_VAR) ) { // full var namespace in resources.
		var varName = value.substr(RDFTemplate.NS_VAR.length);
		var aVar = this._vars[varName];
		if (!aVar) {
			aVar = new RDFTemplate.Var(varName);
			this._vars[varName] = aVar;
		}
		return aVar;
	}
	return null;
}

RDFTemplate.prototype.setVar = function
(

	varName,
	value
)
{
	aVar = this._vars[varName];
	if (!aVar)
		throw "Variable " + varName + " is not defined in this template.";
	aVar.value = value;	
}

RDFTemplate.prototype.setObject = function
(
	theObject
)
{
	var someVar = false;
	for (var varName in this._vars) {
		var aVar = this._vars[varName];
		if (varName in theObject) {
			someVar = true;	
			aVar.value = theObject[varName];
		}
	}
	if (!someVar)
		throw "No defined variables match properties of the supplied object.";
}
RDFTemplate.prototype.output = function
/* Process all triples for the current set of variable binding:
 * 1. rdf:_1 predicates are incremtented for each use.	
 */
(
)
{
	for (var i=0; i < this._triples.length; i++) {
		var triple = this._triples[i];
		var subject = triple.subject;
		var predicate = triple.predicate;
		var object = triple.object;
		
		if (subject instanceof RDFTemplate.Var) {
			if (!subject.value)
				continue;
			if (!(subject.value instanceof Components.interfaces.nsIRDFResource))
				subject = RDF.resource(subject.value);
		}
		
		if (predicate instanceof RDFTemplate.Var) {
			if (!predicate.value)
				continue;
			if (!(predicate.value instanceof Components.interfaces.nsIRDFResource))
				predicate = RDF.resource(predicate.value);
		}
		
		if (object instanceof RDFTemplate.Var) {
			if (object.value == undefined || object.value == null)
				continue;
			if (triple.objIsResource) {
				if (!(object.value instanceof Components.interfaces.nsIRDFResource))
					object = RDF.resource(object.value);
			}
			else {
				object = RDF.node(object.value);
			}
		}
			
		if (subject && predicate && object) {
			if (triple.makeContainer) 
				triple.makeContainer(this, subject);
			else if (triple.appendObject)
				triple.appendObject(this, subject, object);
			else {
				try {
					this._DSOutput.Assert(subject, predicate, object, true);
				}
				catch (e) {
					var i =0;
				/*OK if triple already exists*/}
			}
		}
	}
}
RDFTemplate.prototype.flush = function() {
	this._DSOutput instanceof Components.interfaces.nsIRDFRemoteDataSource;
	this._DSOutput.Flush();
}

RDFTemplate.NS_VAR = "http://xulapp.org/rdf/var#";

RDFTemplate.VAR = "var:";
RDFTemplate.NEXTVAL = "http://www.w3.org/1999/02/22-rdf-syntax-ns#nextVal";
RDFTemplate.INSTANCEOF = "http://www.w3.org/1999/02/22-rdf-syntax-ns#instanceOf";