/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *  Function QueryEngine.execute derived from the inscrutable "establish8"
 *  of Computing with Logic by David Maier and David S. Warren.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
function Collator(NSResolver, RDFResults)
{
	Collator.NSResolver = NSResolver;
	Collator.RDFResults = RDFResults;
	Collator.startDelimiter = NSResolver.startDelimiter;
	Collator.endDelimiter = NSResolver.endDelimiter;
}
 
Collator.Default = function(a, b) {
	if (!a && !b) // Nulls collate together.
		return 0;
	if (!a) return -Collator.order;	
	if (!b) return Collator.order;
	var aIsResource = false;
	var bIsResource = false;
	if (Collator.RDFResults) {
		aIsResource = (a instanceof Components.interfaces.nsIRDFResource);
		bIsResource = (b instanceof Components.interfaces.nsIRDFResource);
		a = RDF.value(a, Collator.NSResolver);
		b = RDF.value(b, Collator.NSResolver);

	}
	else {
		if (typeof(a) == "string")
			aIsResource = (a.startsWith(Collator.startDelimiter) && a.endsWith(Collator.endDelimiter));
		if (typeof(b) == "string")			
			bIsResource = (b.startsWith(Collator.startDelimiter) && b.endsWith(Collator.endDelimiter));
	}
	
	if (aIsResource) {
		if (bIsResource) {
			if (a < b) return Collator.order;
			if (a > b) return -Collator.order;
			return 0;
		}
		else // Resources sort before than literals.
			return Collator.order;
	}
	else {
		if (bIsResource) // Resources sort before than literals.
			return -Collator.order;
		else {
			if (a < b) return Collator.order;
			if (a > b) return -Collator.order;
			return 0;
		}
	}
}

Collator.IgnoreCase = function(a, b) {
	if (!a && !b) // Nulls collate together.
		return 0;
	if (!a) return -Collator.order;	
	if (!b) return Collator.order;
	var aIsResource = false;
	var bIsResource = false;
	if (Collator.RDFResults) {
		aIsResource = (a instanceof Components.interfaces.nsIRDFResource);
		bIsResource = (b instanceof Components.interfaces.nsIRDFResource);
		a = RDF.value(a, Collator.NSResolver);
		b = RDF.value(b, Collator.NSResolver);

	}
	else {
		if (typeof(a) == "string")
			aIsResource = (a.startsWith(Collator.startDelimiter) && a.endsWith(Collator.endDelimiter));
		if (typeof(b) == "string")			
			bIsResource = (b.startsWith(Collator.startDelimiter) && b.endsWith(Collator.endDelimiter));
	}
	a = a.toString().toLowerCase();
	b = b.toString().toLowerCase();
	
	if (aIsResource) {
		if (bIsResource) {
			if (a < b) return Collator.order;
			if (a > b) return -Collator.order;
			return 0;
		}
		else // Resources sort before than literals.
			return Collator.order;
	}
	else {
		if (bIsResource) // Resources sort before than literals.
			return -Collator.order;
		else {
			if (a < b) return Collator.order;
			if (a > b) return -Collator.order;
			return 0;
		}
	}
}

Collator.Numeric = function(a, b) {
	if (!a && !b) // Nulls collate together.
		return 0;
	if (!a) return -Collator.order;	
	if (!b) return Collator.order;
	var aIsResource = false;
	var bIsResource = false;
	if (Collator.RDFResults) {
		aIsResource = (a instanceof Components.interfaces.nsIRDFResource);
		bIsResource = (b instanceof Components.interfaces.nsIRDFResource);
		a = RDF.value(a, Collator.NSResolver);
		b = RDF.value(b, Collator.NSResolver);

	}
	else {
		if (typeof(a) == "string")
			aIsResource = (a.startsWith(Collator.startDelimiter) && a.endsWith(Collator.endDelimiter));
		if (typeof(b) == "string")			
			bIsResource = (b.startsWith(Collator.startDelimiter) && b.endsWith(Collator.endDelimiter));
	}

	if (aIsResource) {
		if (bIsResource) {
			if (a < b) return Collator.order;
			if (a > b) return -Collator.order;
			return 0;
		}
		else // Resources sort before than literals.
			return Collator.order;
	}
	else {
		if (bIsResource) // Resources sort before than literals.
			return -Collator.order;
		else {
			try {
				a = parseFloat(a);
				b = parseFloat(b);
			}
			catch (e) {}

			if (a < b) return Collator.order;
			if (a > b) return -Collator.order;
			return 0;
		}
	}
}