/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *  Function QueryEngine.execute derived from the inscrutable "establish8"
 *  of Computing with Logic by David Maier and David S. Warren.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */

XaLib.include("chrome://xalib/content/RDF/RDF.js");
XaLib.include("chrome://xalib/content/RDF/SPARQL/QueryCompile.js");
XaLib.include("chrome://xalib/content/RDF/SPARQL/QueryEngine.js");


function Query(spec)
{
	this._spec = spec;
	this._queryEngine = new QueryEngine(this);
}
Query.getMessage = function getMessage() {
	if (!Query._messages)
		Query._messages = new MessageFile("chrome://xalib/locale/RDF/SPARQL/Query.msg");

	return Query._messages.getMessage(arguments);
}

Query.prototype.__defineGetter__('spec', function () {
	return this._spec;	
})

function QueryEngine(query)
{
	this._query = query;
	this._spec = query._spec;
	this._dataStore = null;
}

function Var
(
	name,
	value
)
{
	this.type = QueryEngine.VAR;
	this.name = name;
	this.value = value;
	// Value set explicitely by user.
	this.isExplicitValue = (this.value != undefined);
}
function Asc(aVar, collatorFun) {
	this.type = Query.ASC;
	if (arguments.length == 0)
		return;
	if (!(aVar instanceof Var)) {
		throw Query.getMessage("Compile.AscNotVar", aVar);
	}
	if (collatorFun && typeof(collatorFun) != "function") {
			throw Query.getMessage("Compile.AscNotCollator", aVar);
	}
	var self = new Asc();
	self.Var = aVar;
	if (collatorFun != undefined)
		self.collatorFun = collatorFun;
	return self;
}


function Desc(aVar, collatorFun) {
	this.type = Query.DESC;
	if (arguments.length == 0)
		return;
	if (!(aVar instanceof Var)) {
			throw Query.getMessage("Compile.DescNotVar", aVar);
	}
	if (collatorFun && typeof(collatorFun) != "function") {
		throw Query.getMessage("Compile.DescNotCollator", aVar);
	}

	var self = new Desc();
	self.Var = aVar;
	if (collatorFun != undefined)
		self.collatorFun = collatorFun;
	return self;
}


/** Query elements (Group, Triple etc.)
* 1. Are self constructing. i.e. a call to Group(...) yields a new Group object.
* 2. They also self assemble into a tree structure. Siblings are linked by nextItem chains.
*    and descendants are accessible from the childItem property.
*/
function Group()
// A group of patterns to be AND'ed together
{
	this.type = Query.GROUP;
	if (arguments.length == 0)
		return;
	var args = arguments;	
	// handle special call to make master group which passes a single array.
	if (arguments.length == 1 && arguments[0] instanceof Array)
		args = arguments[0];	
	var self	= new Group();
	self.childItem = Query.makeList(self, args);
	return self;
}

function Optional()
// A group of patterns which is allowed to fail.
{
	this.type = Query.OPTIONAL;
	this.isBranching = true; // Does this Term generate alternate code sequences	
	if (arguments.length == 0)
		return;
	var self = new Optional();	
	self.childItem = Query.makeList(self, arguments);
	return self;
}

function Union()
// A group of patterns which are OR'ed together.
{
	this.type = Query.UNION;
	this.isBranching = true; // Does this Term generate alternate code sequences
	if (arguments.length == 0) // see apparently recursive use below - just a constructor.
		return;
	var self = new Union();			
	var	i = arguments.length;	
	//Convert into chain Union.childItem = args[0]
	//             			  .nextAlt  = Union.nextGoal = args[1] etc.  
	var aSubGoal;
	self.nextAlt = null;
	self.childItem = arguments[0];
	while (--i>0) {
		aSubGoal = new Union();
		aSubGoal.childItem = arguments[i];
		aSubGoal.childItem.parent = self;
		aSubGoal.nextAlt = self.nextAlt;
		self.nextAlt = aSubGoal;
	}
	return self;
}

function Graph()
// Access to named patterns.
{
	this.type = Query.GRAPH;
	if (arguments.length == 0)
		return;
	var self = new Graph();
	self.graph = arguments[0];
	// Althought patArray is an array it is derived from arguments and does not 
	// support the usual Array functions like shift.
	var args = [];
	for (var i=1; i<arguments.length; i++)
		args[i - 1] = arguments[i]
	
	self.childItem = Query.makeList(self, args);
	return self;
}

function Resources(subject) {
	this.type = Query.RESOURCES;
	if (arguments.length == 0)
		return;
	var self = new Resources();
	self.subject = subject;
	return self;
}

function ArcsIn(object, predicate) {
	this.type = Query.ARCS_IN;
	if (arguments.length == 0)
		return;
	var self = new ArcsIn();
	self.object = object;
	self.predicate = predicate;
	return self;
}

function ArcsOut(subject, predicate) {
	if (arguments.length == 0)
		return;
	var self = new ArcsOut();
	self.type = Query.ARCS_OUT;
	self.subject = subject;
	self.predicate = predicate;	
	return self;
}

function Triple(subject, predicate, object)
{
	this.type = Query.TRIPLE;
	if (arguments.length == 0)
		return;
	var self = new Triple();
	self.subject = subject;
	self.predicate = predicate;
	self.object = object;
	return self;
}

function In(patArray)
/* This function accepts any number of entries and a final Var.
* The Var binds to each of the entries in turn. 
* At run time the entries must be bound.
* If the Var is bound <tt>In</tt> succeeds if the bound value is in the list.
* Otherwise Var is bound to each of the entries in turn.
*/
{
	this.type = Query.IN;
	if (arguments.length == 0)
		return;
	var self = new In();
	self.entries = [];
	for (var i=0; i<arguments.length - 1; i++)
		self.entries[i] = arguments[i]
	self.retVar = arguments[arguments.length - 1];
	return self;
}
function Builtin
// Test if <tt>op1</tt> has relationship <tt>operator</tt> to <tt>op2</tt>.
// If any required operands are unbound the result is false.
(
	name,	//Builtin operation name.
	argsArray //arguments.
)
{
	this.type = Query.BUILTIN;
	this.name = name;
	this.args = argsArray;
}

function Filter
// Test if <tt>op1</tt> has relationship <tt>operator</tt> to <tt>op2</tt>.
// If any required operands are unbound the result is false.
(
	op1,	//oneof(Var, nsIRDFNode): must be bound.
	opr, 	//string: the operator ==,!== ,< ,<= ,> ,>=,=~,!~
	op2 	//oneof(Var, nsIRDFNode): must be bound.
) //boolean:
{
	return new Builtin("Filter", arguments);
}

function Fun
// This evaluates the function and returns the result.
// If any required operands are unbound the result remains unbound and is false is returned.
(
	op1,	//oneof(Var, nsIRDFNode): must be bound.
	opr, 	//string: the operator "+","-" etc.
	op2,	//oneof(Var, nsIRDFNode): must be bound.
	result	//Var: must be unbound.
) 			//boolean:
{
	return new Builtin("Fun", arguments);
}
function IsBound
// This checks if the operand is a bound variable.
// If any required operands are unbound the result remains unbound and is false is returned.
(
	op1		//oneof(Var, nsIRDFNode): 
) 			//boolean:
{
	return new Builtin("IsBound", arguments);

}
function EndMarker
// A marker which is placed at the end of Optional and Or groups
// It is used to signal at least one success of the group's contents.
(
	refItem
)
{
	this.type = Query.END_MARKER;
	this.refItem = refItem;
}

Query.prototype.execute = function
(
)
{
	this._results = [];
	this._queryEngine.execute(this);
}

Query.prototype.results = function
(
)
{
	var i = 0;
	var self = this;
	var iterator = function () {
		if (i < self._results.length)
			return self._results[i++];
		else {
			self = null;
			return null;
		}
	}
	return iterator;
}
Query.prototype.__defineGetter__('resultsArray', function () {
	return this._results;	
})

Query.makeList = function(parent, args) {
	var itemList = null;
	var	i = args.length; 
	if (!i)
		throw Query.getMessage("Compile.EmptyElement", parent.type);
		
	if (parent.type == Query.OPTIONAL) {
		itemList = new EndMarker(parent);
	}
	
	//convert to linked list.
	while  (--i>-1) {
		var item = args[i];
		item.parent = parent;
		item.nextItem = itemList;
		itemList = item;
	}
	return itemList;
}


Query.VAR = "Var";
Query.DESC = "Desc";
Query.ASC = "Asc";
Query.GROUP = "Group";
Query.OPTIONAL = "Optional";
Query.UNION = "Union";
Query.GRAPH = "Graph";
Query.TRIPLE = "Triple";
Query.TRIPLE_SPO = "SPO";
Query.TRIPLE_SP$ = "SP$";
Query.TRIPLE_$PO = "$PO";
Query.RESOURCES = "R";
Query.ARCS_IN = "AI";
Query.ARCS_OUT = "AO";
Query.IN = "In";
Query.BUILTIN = "B";
Query.END_MARKER = "End";
Query.ONCE = "Once";