/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Peter Wilson, orignal author.
 *  Function QueryEngine.execute derived from the inscrutable "establish8"
 *  of Computing with Logic by David Maier and David S. Warren.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */

XaLib.include("chrome://xalib/content/RDF/RDF.js");
//XaLib.include("chrome://xalib/content/RDF/SPARQL/Query.js");
XaLib.include("chrome://xalib/content/RDF/SPARQL/QueryCompile.js");
XaLib.include("chrome://xalib/content/RDF/SPARQL/Collator.js");

QueryEngine.Frame = function() {
}

QueryEngine.prototype.execute = function
(
	query
)
{
	var spec = query._spec;
	if (spec.Distinct == true)
		this._DistinctMap = {}; // A map to store record hashes. Yes this will hurt performance.
	else
		this._DistinctMap = null;
	
	this.NSResolver = query.NSResolver;
	var results = query._results;
	var limit = query._Limit;
	if (limit == 0) // Always empty reslt.
		return true;
	this._dataStores = [];
	this._dataStores.push(query._dataStore);
	
	var curFrame = new QueryEngine.Frame();
	curFrame.goal = query._topGoal;
	curFrame.nextAlt = null;

	var aGoal;	
	var curGoal;
	var lastBack = curFrame;
nextGoal:
	do {
		curGoal = curFrame.goal;

nextAlt:do {
			if (curGoal) {
				if (curGoal.execute(this, curFrame)) {
					var nextGoal = curGoal.nextGoal;
					if (nextGoal) {
						if (curFrame.nextAlt)
							lastBack = curFrame;
						if (curFrame.nextFrame) { // reuse Frames.
							nextFrame = curFrame.nextFrame;
							nextFrame.nextAlt = null;
						}	
						else {	
							nextFrame = new QueryEngine.Frame();
							nextFrame.parent = curFrame;
							curFrame.nextFrame = nextFrame;
						}	
						nextFrame.back = lastBack;
						nextFrame.goal = nextGoal;
						curFrame = nextFrame;
						continue nextGoal;
					}
					else {
						// we've reached the end of the goal list so produce a result.
						var result = this.buildResult();
						if (result)
							results.push(result);
						// Check Limit clause.	
						if (spec.OrderBy == undefined && results.length >= limit) {
							this._DistinctMap = null;
							return true;
						}
					}
				}
			}
			else {
				curFrame = lastBack;
				if (!curFrame) {
					if (spec.OrderBy != undefined)
						this.sortResults(results);
					this._DistinctMap = null;
					return true;
				}
				lastBack = curFrame.back;
			}
			
			if (!curFrame) {
				this._DistinctMap = null;
				return false;
			}

			aGoal = curFrame.goal;
			if (aGoal && aGoal.restore)	
				aGoal.restore(curFrame);	
			curGoal = curFrame.nextAlt;
		} while(true);
	} while (true);
}

QueryEngine.prototype.buildResult = function
(
)
{
	var result = {};
	var spec = this._spec;
	var hash = 0;
	

	// Distinct values are checked by computing a hash of the result column values
	var isDistinct = (this._DistinctMap != null);
	
	// for each variable in the Select list.
	for (var i=0; i<spec.Select.length; i++) {
		var aVar = spec.Select[i];
		if (this.RDFResults) { // Raw RDF values required.
			var val = this.deref(aVar).value;
			result[aVar.name] = val;
			if (isDistinct) {
			   	if (hash > 9999999999999) // Maximum number represented as an integer.
			   		hash = hash % 751229437; // some arbitrary divisor to stop the hash getting too large.
				hash = (hash*31) + val.Value.hash();
			}
		}	
		else { // literal values required.
			val = RDF.value(this.deref(aVar).value, this.NSResolver);
			result[aVar.name] = val;
			if (isDistinct) { 	
			   	if (hash > 9999999999999) // Maximum number represented as an integer.
			   		hash = hash % 751229437; // some arbitrary divisor to stop the hash getting too large.
				hash = (hash*31) + val.hash();
			}
		}
	}
	if (isDistinct) {
		//FIXME If two rows have the same hash value they are PRESUMED to be equal.
		// hash is 10^15 so we assume a collision is a duplicate.
		// But, Hey!
		var aKey = "X" + hash;
		if (this._DistinctMap[aKey] == undefined) {
			this._DistinctMap[aKey] = true // store "result" if you want to check field values;
		}
		else {
			result = null;
			// You could check each field of result and the previous result.
		}
	}
	return result;
}

QueryEngine.prototype.sortResults = function
(
	results
)
{
	var orderBy = this._spec.OrderBy;
	var orderByLen = orderBy.length;
	var comparator = function (a, b)
	{
		for (var i=0; i<orderByLen; i++) {
			var sortTerm = orderBy[i];
			var varName = sortTerm.Var.name;  // Asc(var) or Desc(var) with optional ignorecase.
			var collatorFun = sortTerm.collatorFun;
			if (!collatorFun)
				collatorFun = Collator.Default;
			Collator.order = (sortTerm.type == Query.ASC)?-1:1; //This is evil.
			var cmp = collatorFun(a[varName], b[varName]);
			if (cmp) return cmp;
		}
		return 0;
	}
	
	results.sort(comparator);
	comparator = null;
	// Apply Limit clause:
	if (this._spec.Limit) {
		while (results.length > this._spec.Limit)
			results.pop();
	}
}

QueryEngine.prototype.setDataStores = function
(
	DSArray
)
{
	//TODO allow nsIDataSource or string URL
	if (DSArray.length > 1) {
		DS = Components.classes["@mozilla.org/rdf/datasource;1?name=composite-datasource"]
        .createInstance(Components.interfaces.nsIRDFCompositeDataSource);
		for (var i=0; i<DSArray.length; i++) {
			var innerDS = DSArray[i];
			if (!(innerDS instanceof Components.interfaces.nsIRDFDataSource))
				innerDS = RDF.getDataSource(innerDS);
			DS.addDataSource(innerDS);	
		}
	}
	else {
		var DS = DSArray[0];
		if (typeof(DS) == "string")
			DS = RDF.getDataSource(DS);
		this._dataStore = DS;
	}	
}
QueryEngine.prototype.initVars = function
(
	varArray // from the Select clause.
)
{
	//build a map of all variable in the Select clause.
	//Note: Additional Vars may be added by other goals.
	for (var i=0; i<varArray.length; i++) {
		var aVar = varArray[i];
		this.addVarIf(aVar);
	}
}
QueryEngine.prototype.newTempVar = function () {
	this._tempVarCount++;
	var aVar = new Var("_X" + this._tempVarCount);
	this.addVarIf(aVar);
	return aVar;
}

// Groups are compiled out.
//QueryEngine.Group = function () {}/////////////////////////////////////////////

QueryEngine.Optional = function (nextGoal, nextAlt) {
	this.type = Query.OPTIONAL;
	this.nextGoal = nextGoal; // The set of goals when the option group succeeds.
	this.nextAlt = nextAlt; // The set of goals to be persued when the Optional fails.
}

QueryEngine.Optional.prototype.execute = function
(
	queryEngine,
	frame
)
{
	this.nextAlt.failed = true; // Assume for now.
	frame.nextAlt = this.nextAlt; // cause this to be invoked on failure.
	return true;
}

// An EndMarker is inserted after the last goal in an Optional Group.
// Its job is to signal the Optional that the optional succeeded at least once.
QueryEngine.EndMarker = function (refGoal) {
	this.type = Query.END_MARKER;
	this.refGoal = refGoal;
}
QueryEngine.EndMarker.prototype.execute = function
(
	queryEngine,
	frame
)
{
	this.refGoal.nextAlt.failed = false;
	return true;
}
// A goal which succeeds once.
QueryEngine.Once = function () {
	this.type = Query.ONCE;
}
QueryEngine.Once.prototype.execute = function
(
	queryEngine,
	frame
)
{
	frame.nextAlt = null;
	if (this.failed) {
			return true;
	}
	else {
		this.failed = true;			
		return false;
	}	
}


QueryEngine.Union = function (nextGoal, nextAlt) 
{
	this.type = Query.UNION;
	this.nextGoal = nextGoal;
	this.nextAlt = nextAlt;
}

QueryEngine.Union.prototype.execute = function
(
	queryEngine,
	frame
)
{
	frame.nextAlt = this.nextAlt;;
	return true;

}

QueryEngine.Graph = function (graph, goals) {
	this.type = Query.GRAPH;
	this.graph = graph;
	this.nextGoal = goals;
}

QueryEngine.Graph.prototype.execute = function
(
	queryEngine,
	frame
)
{
	var graph = this.graph;
	if (graph instanceof Var) { 
		graph = queryEngine.deref(graph).value;
	}
	if (graph != undefined) { // Already bound.
		if (frame.nextAlt == undefined) {
			if (!(graph instanceof Components.interfaces.nsIRDFLiteral))
				throw Query.getMessage("Engine.GraphLiteral", graph);			
			var DS = RDF.getDataSource(graph.Value);
			queryEngine._dataStores.push(DS);
			frame.nextAlt = this; // This gives opportunity to remove the DS on call back.
			return true;
		}
		else {
			queryEngine._dataStores.pop(DS);
			frame.nextAlt = undefined;
			return false;
		}
	}
	// graph is unbound variable and should iterate thru FromNamed DataStores.
	if (frame.itr == undefined) {
		frame.itr = 0;
	}
	var FromNamed = queryEngine._spec.FromNamed;
	if (frame.itr < FromNamed.length) {
		var DS = FromNamed[frame.itr];
		// Replace top datastore with new one.
		if (frame.itr > 0)
			queryEngine._dataStores.pop();
		queryEngine._dataStores.push(DS);
		this.gBinding = this.graph;
		this.gBinding.value = DS.URI;
		frame.itr++;
		frame.nextAlt = this;
		return true;
	}
	queryEngine._dataStores.pop();
	frame.itr = undefined;
	frame.nextAlt = null;
	return false;
}
QueryEngine.Graph.prototype.restore = function
(
	frame
)
{
	if (this.gBinding)
		this.gBinding.value = undefined;
}


QueryEngine.Resources = function (subject) {
	this.type = Query.RESOURCES;
	this.subject = subject;
	if (subject instanceof Var)
		this.sBinding = subject;
}
QueryEngine.Resources.prototype.execute = function
// Iterate through all the resources in the DataStore.
(
	queryEngine,
	frame
)
{
	var subject = this.subject;
	if (subject instanceof Var) { 
		this.sBinding = queryEngine.deref(subject);
		subject = this.sBinding.value;
	}
	if (subject != undefined) { // Ground. 
		//TODO Iterate through resources and match then return once or empty iterator.
	}
	else {
		var itr = frame.itr;
		if (!itr) {
			var DS = queryEngine._dataStores;
			DS = DS[DS.length - 1];
			frame.itr = DS.GetAllResources();
			itr = frame.itr;
		}
	
		if (itr) {
			if (itr.hasMoreElements()) {
				var subject = itr.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
				this.sBinding.value = subject;
				frame.nextAlt = this;
				return true;
			}
			else {
				this.sBinding.value = undefined;
				frame.itr = null;
				frame.nextAlt = null;
				return false;
			}
		}
		else	
			return false;
	}
}
QueryEngine.Resources.prototype.restore = function
(
	frame
)
{
	if (this.sBinding)
		this.sBinding.value = undefined;
}
QueryEngine.ArcsIn = function (object, predicate) {
	this.type = Query.ARCS_IN;
	this.object = object;
	this.predicate = predicate;
	if (predicate instanceof Var)
			this.pBinding = predicate;
}
QueryEngine.ArcsIn.prototype.execute = function
// Iterate through all the predicates arcs into a given object.
// Only unique predicates are returned.
(
	queryEngine,
	frame
)
{
	var object = this.object;
	var predicate = this.predicate;
	if (object instanceof Var) { 
		object = queryEngine.deref(object).value;
	}
	if (predicate instanceof Var) { 
		this.pBinding = queryEngine.deref(predicate);
		predicate = this.pBinding.value;
	}
	
	if (object == undefined) 
		throw Query.getMessage("Engine.ArcsInUnbound", this.object.name);			

	if (predicate != undefined) { // Ground. 
		//Iterate through predicates looking for current predicate.
		var DS = queryEngine._dataStores;
		DS = DS[DS.length - 1];
		var itr = DS.ArcLabelsIn(object);
		while (itr && itr.hasMoreElements()) {
			var pred = itr.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
			if (pred == predicate) {
				this.pBinding.value = predicate;
				frame.nextAlt = null;
				return true;
			}
		}
		this.pBinding.value = undefined;
		frame.nextAlt = null;
		return false;
	}
	// otherwise predicate is unbound variable:
	var itr = frame.itr;
	if (!itr) {
		var DS = queryEngine._dataStores;
		DS = DS[DS.length - 1];
		frame.itr = DS.ArcLabelsIn(object);
		itr = frame.itr;
	}

	if (itr) {
		if (itr.hasMoreElements()) {
			var predicate = itr.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
			this.pBinding.value = predicate;
			frame.nextAlt = this;
			return true;
		}
		else {
			this.pBinding.value = undefined;
			frame.itr = null;
			frame.nextAlt = null;
			return false;
		}
	}
	else
		return false;
}
QueryEngine.ArcsIn.prototype.restore = function
(
	frame
)
{
	if (this.pBinding)
		this.pBinding.value = undefined;
}

QueryEngine.ArcsOut = function (subject, predicate) {
	this.type = Query.ARCS_OUT;
	this.subject = subject;
	this.predicate = predicate;
	if (predicate instanceof Var)
		this.pBinding = predicate;
}
QueryEngine.ArcsOut.prototype.execute = function
// Iterate through all the predicates for a given object.
(
	queryEngine,
	frame
)
{
	var subject = this.subject;
	var predicate = this.predicate;
	if (subject instanceof Var) { 
		subject = queryEngine.deref(subject).value;
	}
	if (predicate instanceof Var) { 
		this.pBinding = queryEngine.deref(predicate);
		predicate = this.pBinding.value;
	}
	
	if (subject == undefined) 
		throw Query.getMessage("Engine.ArcsOutUnbound", this.subject.name);			

	if (predicate != undefined) { // Ground. 
		//Iterate through predicates looking for current predicate.
		var DS = queryEngine._dataStores;
		DS = DS[DS.length - 1];
		var itr = DS.ArcLabelsOut(subject);
		while (itr && itr.hasMoreElements()) {
			var pred = itr.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
			if (pred == predicate) {
				this.pBinding.value = predicate;
				frame.nextAlt = null;
				return true;
			}
		}
		this.pBinding.value = undefined;
		frame.nextAlt = null;
		return false;
	}
	// otherwise predicate is unbound variable:
	var itr = frame.itr;
	if (!itr) {
		var DS = queryEngine._dataStores;
		DS = DS[DS.length - 1];
		frame.itr = DS.ArcLabelsOut(subject);
		itr = frame.itr;
	}

	if (itr) {
		if (itr.hasMoreElements()) {
			var predicate = itr.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
			this.pBinding.value = predicate;
			frame.nextAlt = this;
			return true;
		}
		else {
			this.pBinding.value = undefined;
			frame.itr = null;
			frame.nextAlt = null;
			return false;
		}
	}
	else
		return false;
}
QueryEngine.ArcsOut.prototype.restore = function
(
	frame
)
{
	if (this.pBinding)
		this.pBinding.value = undefined;
}

QueryEngine.Triple_SPO = function
// A triple specialized for where subject, predicate and object are bound.
(subject, predicate, object) 
{
	this.type = Query.TRIPLE_SPO;
	// no binding required.
	this.subject = subject;
	this.predicate = predicate;
	this.object = object;
}

QueryEngine.Triple_SPO.prototype.execute = function
(
	queryEngine,
	frame
)
{
	var subject = (this.subject instanceof Var)?queryEngine.deref(this.subject).value:this.subject;
	var predicate = (this.predicate instanceof Var)?queryEngine.deref(this.predicate).value:this.predicate;
	var object = (this.object instanceof Var)?queryEngine.deref(this.object).value:this.object;

	var DS = queryEngine._dataStores;
	DS = DS[DS.length - 1];
	if (DS.HasAssertion(subject, predicate, object, true)) {
		frame.nextAlt = null;
		return true;
	}
	else {
		frame.nextAlt = null;
		return false;
	}
		
}

QueryEngine.Triple_SP$ = function
// A triple specialized so that only object is variable.
(subject, predicate, object) 
{
	this.type = Query.TRIPLE_SP$;
	this.oBinding = this.predicate; // a Var
	this.subject = subject;
	this.predicate = predicate;
	this.object = object;
}

QueryEngine.Triple_SP$.prototype.execute = function
(
	queryEngine,
	frame
)
{
	var subject = (this.subject instanceof Var)?queryEngine.deref(this.subject).value:this.subject;
	var predicate = (this.predicate instanceof Var)?queryEngine.deref(this.predicate).value:this.predicate;
	var object = this.object;
	if (object instanceof Var) { 
		this.oBinding = queryEngine.deref(object);
		object = this.oBinding.value;
	}

	if (object != undefined) // SPO all ground. 
		return QueryEngine.Triple_SPO(subject,predicate,object);
	

	// otherwise object is unbound variable:
	var itr = frame.itr;
	if (!itr) {
		var DS = queryEngine._dataStores;
		DS = DS[DS.length - 1];
		frame.itr = DS.GetTargets(subject, predicate, true);
		itr = frame.itr;
	}

	if (itr) {
		if (itr.hasMoreElements()) {
			var object = RDF.QINode(itr.getNext());
			this.oBinding.value = object;
			frame.nextAlt = this;
			return true;
		}
		else {
			this.oBinding.value = undefined;
			frame.itr = null;
			frame.nextAlt = null;
			return false;
		}
	}
	else
		return false;
}
QueryEngine.Triple_SP$.prototype.restore = function
(
	frame
)
{
	if (this.oBinding)
		this.oBinding.value = undefined;
}

QueryEngine.Triple_$PO = function
// A triple specialized so that only subject is variable.
(subject, predicate, object) 
{
	this.type = Query.TRIPLE_$PO;
	this.sBinding = this.predicate; // a Var
	this.subject = subject;
	this.predicate = predicate;
	this.object = object;
}

QueryEngine.Triple_$PO.prototype.execute = function
(
	queryEngine,
	frame
)
{
	var predicate = (this.predicate instanceof Var)?queryEngine.deref(this.predicate).value:this.predicate;
	var object = (this.object instanceof Var)?queryEngine.deref(this.object).value:this.object;
	var subject = this.subject;
	if (subject instanceof Var) { 
		this.sBinding = queryEngine.deref(subject);
		subject = this.sBinding.value;
	}

	if (subject != undefined) // SPO all ground. 
		return QueryEngine.Triple_SPO(subject,predicate,object);

	// otherwise subject is unbound variable:
	var itr = frame.itr;
	if (!itr) {
		var DS = queryEngine._dataStores;
		DS = DS[DS.length - 1];
		frame.itr = DS.GetSources(predicate, object, true);
		itr = frame.itr;
	}

	if (itr) {
		if (itr.hasMoreElements()) {
			var subject = itr.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
			this.sBinding.value = subject;
			frame.nextAlt = this;
			return true;
		}
		else {
			this.sBinding.value = undefined;
			frame.itr = null;
			frame.nextAlt = null;
			return false;
		}
	}
	else
		return false;
}
QueryEngine.Triple_$PO.prototype.restore = function
(
	frame
)
{
	if (this.sBinding)
		this.sBinding.value = undefined;
}
QueryEngine.In = function (retVar, entries) {
	this.type = Query.IN;
	this.retVar = retVar;
	this.entries = entries;
}
QueryEngine.In.prototype.execute = function
// Iterate through all the resources in the <tt>In</tt> function.
(
	queryEngine,
	frame
)
{
	var retVar = this.retVar;
	if (retVar instanceof Var) { 
		this.rBinding = queryEngine.deref(retVar);
		retVar = this.rBinding.value;
	}
	if (retVar != undefined) { // Ground. 
		frame.nextAlt = null;
		//Iterate through entries and match them return once.
		for (var i=0; i<this.entries; i++) {
			var entry = this.entries[i];
			var entry = (entry instanceof Var)?queryEngine.deref(entry).value:entry;
			if (entry == retVar)
				return true;
		}
		return false;
	}
	// retVar is unbound

	// itr is an index into the In.entries.
	if (frame.itr == undefined) {
		frame.itr = 0;
	}

	if (frame.itr < this.entries.length) {
		var entry = this.entries[frame.itr++];
		if (entry instanceof Var) { 
			var aVar = queryEngine.deref(entry);
			if (AVar.value == undefined)
				throw Query.getMessage("Engine.InUnbound", this.entry.name);			
			entry = aVar.value;
		}
		this.rBinding.value = entry;
		frame.nextAlt = this;
		return true;
	}
	else {
		this.rBinding.value = undefined;
		frame.itr = undefined;
		frame.nextAlt = null;
		return false;
	}
}
QueryEngine.In.prototype.restore = function
(
	frame
)
{
	if (this.rBinding)
		this.rBinding.value = undefined;
}

QueryEngine.Builtin = function (name, retVar, args) {
	this.type = Query.BUILTIN;
	this.name = name;
	this.retVar = retVar;
	this.args = args;
}

QueryEngine.Builtin.prototype.execute = function
(
	queryEngine,
	frame
) // boolean:
{
	var opr = this.args[1];
	switch (this.name) {
	case "Filter":
	case "Fun":
		var op1 =  this.args[0];
		if (op1 instanceof Var) {
			op1 = queryEngine.deref(op1).value;
			if (op1 == undefined)
				return false; // spec says no match
		}

		var op2 =  this.args[2];	
		if (op2 instanceof Var) {
			op2 = queryEngine.deref(op2).value;
			if (op2 == undefined)
				return false; // spec says no match
		}
		if (this.name == "Filter") {
			switch (opr) {
			case "==":
				return (op1 == op2);
			case "!=":
				return (op1 != op2);
			case "<":
				return (op1.Value < op2.Value);
			case "<=":
				return (op1.Value <= op2.Value);
			case ">":
				return (op1.Value > op2.Value);
			case ">=":
				return (op1.Value >= op2.Value);
			case "=~":
				if (!(op2 instanceof RegExp))
					throw Query.getMessage("Engine.Filter1", op2);			
				return op2.test(op1.Value);
			case "!~":
				if (!(op2 instanceof RegExp))
					throw Query.getMessage("Engine.Filter2", op2);			
				return !op2.test(op1.Value);				
			default:
				throw Query.getMessage("Engine.Filter3", opr);			
			}
		}
		// Assume Fun.
		var retVar = this.retVar;
		if (retVar instanceof Var) { 
			this.rBinding = queryEngine.deref(retVar);
			retVar = this.rBinding.value;
			if (retVar != undefined)
				return false; // Already Bound so fail.
		}
		var res;
		
		op1 = op1.Value;
		
		if (opr == "match") {
			if (!(op2 instanceof RegExp))
				throw Query.getMessage("Engine.Fun1", op2);			
			var arr = op2.exec(op1);
			if (arr) {
				this.rBinding.value = RDF.node(arr[0], queryEngine.NSResolver);
				return true;
			}
			else {
				this.rBinding.value = undefined;
				return false;
			}
		}
		
		op2 = op2.Value;
		if (opr == "makeResource") {
			res = op1.toString() + op2.toString();
			res = RDF.resource(res);
			this.rBinding.value =res;
			return true;
		}

		if (opr == "concat")
			res = op1.toString() + op2.toString();
		else {	// Numeric ops.
			if (typeof(op1) != "number") op1 = parseInt(op1);
			if (typeof(op2) != "number") op2 = parseInt(op2);
			switch (opr) {
			case "+":
				res = op1 + op2;
				break;
			case "-":
				res = op1 - op2;
				break;
			case "*":
				res = op1 * op2;
				break;
			case "/":
				res = op1 / op2;
				break;
			case "%":
				res = op1 % op2;
				break;				
			default:
				throw Query.getMessage("Engine.Fun2", opr);						
			}
			if (isNaN(res))
				res = "NaN";
			else	
				if (!isFinite(res))
					res = "Inf";
		}
		this.rBinding.value = RDF.node(res, queryEngine.NSResolver);
		return true;
	case "IsBound":
		var op1 =  this.args[0];
		if (op1 instanceof Var) {
			op1 = queryEngine.deref(op1).value;
			if (op1 == undefined)
				return false; 
		}
		return true;
	default:
		throw Query.getMessage("Engine.Builtin", this.name);						
	}
	return false;
}
QueryEngine.Builtin.prototype.restore = function
(
	frame
)
{
	if (this.rBinding)
		this.rBinding.value = undefined;
}

QueryEngine.prototype.deref = function (aVar) {
	do {
		if (!(aVar.value instanceof Var))
			return aVar;
		aVar = aVar.value;	
	} while (true);
}

