/*
	Based on description of time based uuid's at http://kruithof.xs4all.nl/guid-uuid-timebased.html
	by Martijn Kruithof?
	
	Generally a programmer does not get the current time in 100 nanosecond intervals 
	since 15 October 1582, but for instance in millisecond precision since 1 January 1970. 
	In this case, to come from milliseconds to nanoseconds precision multiply the time
	returned from the system by 10000 and to correct the start date add an offset
	of 122192928000000000. (With other resolutions another multiplier needs to be applied,
	with other start dates another offset needs to be used. 
	In case multiple UUIDs need to be generated at the same time according to the millisecond
	precision this can be done by adding 1 to this interval for for every UUID requested inside
	the same millisecond period. This must not be performed more than 10000 times in a 
	millisecond period.
	
	When more than 1 UUID is needed per 100 nanoseconds multiple node IDs will be needed.	
*/
XaLib.include("chrome://xalib/content/XPCOM/XPCOM.js");

function UUID
// Allocate a time based UUID. As no MAC address is available in Mozilla we use a random nodeId.
(

)

{
	// convert to 100 ns since 15 October 1582.
	var timeMS = new Date().valueOf() * 10000 + 122192928000000000;

	if (UUID.prevTimeMS == -1) {
		var prefService = XPCOM.getService("@mozilla.org/preferences-service;1", "nsIPrefService");
		var branch = prefService.getBranch("xalib.uuid");
		try {
			UUID.node = branch.getCharPref(".node");
		}
		catch (e) {
			var node = (Math.random() * 100000000000000000);
			//n.b. bit &, | or >> only work on 32 bit integers.
			node = "000000000000" + node.toString(16);
			node = node.substr(node.length - 12);
			// Set high order bit in first hex digit.
			node = UUID.map[node.substr(0,1)] + node.substr(1);
			UUID.node = node;
			branch.setCharPref(".node", UUID.node);
		}	
		try {
			UUID.clock = branch.getIntPref(".clock");
			//Time warp advance clock id.
			if (UUID.prevTimeMS > timeMS) {
				UUID.clock++;
				branch.setIntPref(".clock", UUID.clock);
			}

		}
		catch (e) {
			UUID.clock = (Math.random() * 100000000000000000) & 0xFFFF;
			branch.setIntPref(".clock", UUID.clock);
		}
	}

	var timestamp = timeMS;
	
	// Trap multiple uses within same milliseconds.
	if (UUID.prevTimeMS == timeMS) {
		UUID.dupCount++;
		if (UUID.dupCount > 9999)
			throw "Too many UUID operations (10,000) for same millisecond.";
		timestamp += UUID.dupCount; // Should trap 10,000 uses and throw.
	}
	else {
		UUID.prevTimeMS = timeMS;
		UUID.dupCount = 0;
	}
	
    timestamp = "0000000000" + timestamp.toString(16);
    var tsLength = timestamp.length;
	var uuid = timestamp.substr(tsLength - 8);
	uuid += "-";
	uuid += timestamp.substr(tsLength - 12, 4);
	uuid += "-";
	uuid += "1"; // Type
	uuid += timestamp.substr(tsLength - 15, 3);
	uuid += "-";
	var clock = UUID.clock.toString(16);
	clock = clock.substr(clock.length - 4);
	clock = UUID.map[clock.substr(0,1)] + clock.substr(1);
	uuid += clock;
	uuid += "-";
	uuid += UUID.node;
	return uuid;

}
UUID.prevTimeMS = -1;
UUID.dupCount = 0;
UUID.clockId = -1;
UUID.node;
UUID.map = {};
UUID.map["0"] = "8"; 
UUID.map["1"] = "9"; 
UUID.map["2"] = "a"; 
UUID.map["3"] = "b"; 
UUID.map["4"] = "c"; 
UUID.map["5"] = "d"; 
UUID.map["6"] = "e"; 
UUID.map["7"] = "f"; 
UUID.map["8"] = "8";
UUID.map["9"] = "9";
UUID.map["a"] = "a";
UUID.map["b"] = "b";
UUID.map["c"] = "c";
UUID.map["d"] = "d";
UUID.map["e"] = "e";
UUID.map["f"] = "f";
