/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XulApp IDE.
 *
 * The Initial Developer of the Original Code is Synesis Software (USA) Inc.
 *

 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * Synesis Software (USA) Inc. the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 	Author: Peter Wilson, derived from jsLib to support XulApp requirements.
 *	jslib Original Author: Pete Collins <pete@mozdevgroup.com>
 *  jslib Contributor(s): Martin Kutschker <Martin.T.Kutschker@blackbox.net>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
 /* Notes:
 * Variables or methods prefixed by underscore are for internal use only.
 */
netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect"); 
if (typeof(XaLib) == "undefined") {

//==========================================================================================
 function debug(aText) {
   var console = Components.classes['@mozilla.org/consoleservice;1'].
 		getService(Components.interfaces["nsIConsoleService"]);
   console.logStringMessage(aText);
 }

 function assert(test, msg) {
 	if (!test) {
 		var theMsg = "Assertion failed:\n" + msg + "\n" + getStackTrace();
 		dump(theMsg);
 		alert(theMsg);
 	}

 }
 
 function getStackTrace(includeThis) {
 	var maxDepth = 10;
 	try {
		var frame = Components.stack.caller;
		if (!includeThis)
			frame = frame.caller;
		var str = "Called from: ";
		var depth = 0;
		while (frame) {
			if (frame.filename)
				str += frame.filename + ", Line " + frame.lineNumber;
			else
				str += "[No file name]";

			str += " --> ";

			if (frame.name)
				str += frame.name;
			else
				str += "[global]";

			str += "\n";
			if (++depth > maxDepth) {
				str += "Stack Trace stopped at MAX Depth of " + maxDepth + " frames.\n";
				break;
			}
			frame = frame.caller;
		}
		return str;
	}
	catch (e) {
	   	return "Stack Trace Failed.";
	}
}
   

 

 function MessageFile(url) {
 	this.url = url;
 }
 
 /** Retrieve the string associated with the msgId (args[0]) from the StringBundle.
 *	Additional parameters may be passed as message substitution variables.
 *  Each substitution variable has the following format: {n:name} the :name is optionsl.
 *  E.g.  message file entry: "msgId=An example {1:paramA} message {2:paramB} and {1}." 
 *  Will allow substitution of {1...} and {2...} by the 1st and 2nd parameters respectively (counting from 1).
 *  The same substitution parameter may be used more than once.
 */
 MessageFile.prototype.getMessage = function getMessage(args) {
 	if (!this._stringBundle) {
 		// Load on first use.
		try {
			var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"]
										.getService(Components.interfaces.nsIStringBundleService);

			this._stringBundle = stringBundleService.createBundle(this.url); 
		}
		catch (e) {
			var msg = "MessageFile: create StringBundle failed:" + this.url + "\n" + e + "\n" + getStackTrace();
			alert(msg);
		}
 	}
  	try {
 		var msg = this._stringBundle.GetStringFromName(args[0]);
 	}
 	catch (e) {
		throw 'String resource: "' + args[0] + '" not found. ' + e + '\n' + getStackTrace();
 	}
 		
	// message parameter substitution.
	var re;
	for (var i=1; i < args.length; i++) {
		re = new RegExp("\{" + i + "[:]?[^}]*\}", "g");
		if (args[i] == null) 
			msg = msg.replace(re, "null");
		else
			msg = msg.replace(re, args[i]);
	}
	return msg;
 		
}
 
// An XaLib object is never created. The XaLib function object holds all properties and methods.
function XaLib() {
	alert("Do not call the XaLib constructor.");
}
XaLib.getMessage = function getMessage() {
	return XaLib._messages.getMessage(arguments);
}
XaLib.isIncluded = function(URI) {
	if (!URI)
		return false;
	else	
		return (XaLib._includes[URI.toUpperCase()]!= undefined);

}

XaLib.include = function include(URI) {
//WARNING: chrome:/XXX/content/X.js and chrome:/XXX/contents/X.js are not distinguished by
// The chrome registry. Use ONLY one!
	// include must be re-entrant as includes can be nested.
	function loader(URI) {
		if (!URI)
			throw XaLib.getMessage("include.URINotSpecified");
		// Check for already loaded (and short-circuit circular references).
		var ucURI = URI.toUpperCase();
		//var ucURI = ucURI.replace(/[:\/]/g,"_"); //PW I don't think this is necessary
		var storedURI = XaLib._includes[ucURI];
		if (storedURI) {
			if (storedURI != URI) {
				throw XaLib.getMessage("include.URICaseError", URI, getStackTrace());
			}
			return;
		}

		try {
			XaLib._includes[ucURI] = URI;
			//debug("loading:" + URI);
			this._jsLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].getService(Components.interfaces.mozIJSSubScriptLoader);
			this._jsLoader.loadSubScript(URI);
			//debug("done loading:" + URI);				
		}
		catch (e) {
			var lineNumber = "";
			if (e.lineNumber)
				lineNumber = " line:" + e.lineNumber;
			throw XaLib.getMessage("include.IncludeError", URI, e + lineNumber, getStackTrace());
		}
	}
	new loader(URI);
}
//Globals.
try {
	// Initialize XaLib porperties.
	XaLib._messages = new MessageFile("chrome://xalib/locale/xalib.msg");
	XaLib._includes = {}; // HashMap of URIs of successful includes.
	XaLib._includes.__proto__ = null; // Used as HashMap, so prevent access to Object properties.
	XaLib.ensure = XaLib.include; //An alias for inline use.			
}
catch (e) {
	throw 'XaLib.js: failed to initialize'
}

 
} // End if XaLib undefined